@@ -145,21 +145,21 @@ pls_cv <- function(x, y, ncomp,
                    group = NULL,
                    retrieve = TRUE,
                    tune = TRUE,
-                   max_iter = 1, tol = 1e-6, 
+                   max_iter = 1, tol = 1e-6,
                    seed = NULL,
                    modified = FALSE) {
   min_allowed <- (floor(min(ncol(x), nrow(x)) * p)) - 1
-  
+
   if (min_allowed < ncomp) {
     ncomp <- min_allowed
   }
-  
+
   if (modified) {
     algorithm <- "mpls"
   } else {
     algorithm <- "pls"
   }
-  
+
   cv_samples <- sample_stratified(
     y = y,
     p = p,
@@ -168,7 +168,7 @@ pls_cv <- function(x, y, ncomp,
     replacement = FALSE,
     seed = seed
   )
-  
+
   if (method == "wapls" & retrieve & tune) {
     pls_c <- c(min_component, ncomp)
     seq_pls <- min(pls_c):max(pls_c)
@@ -184,7 +184,7 @@ pls_cv <- function(x, y, ncomp,
     search_grid <- matrix(0, 0, 0)
     fit_method <- method
   }
-  
+
   cv_results <- opls_cv_cpp(
     X = x,
     Y = y,
@@ -197,13 +197,13 @@ pls_cv <- function(x, y, ncomp,
     new_x = new_x,
     maxiter = max_iter,
     tol = tol,
-    wapls_grid = search_grid, 
+    wapls_grid = search_grid,
     algorithm = algorithm
   )
-  
+
   val <- NULL
   val$resamples <- cv_samples$hold_out
-  
+
   if (method == "pls") {
     val$cv_results <- data.table(
       npls = 1:ncomp,
@@ -214,7 +214,7 @@ pls_cv <- function(x, y, ncomp,
     )
     best_pls_c <- val$cv_results$npls[which(val$cv_results$rmse_cv == min(val$cv_results$rmse_cv))]
     val$best_pls_c <- best_pls_c
-    
+
     if (retrieve) {
       if (!is.null(weights)) {
         x <- sweep(x, 1, weights, "*")
@@ -227,7 +227,7 @@ pls_cv <- function(x, y, ncomp,
           scale = scale,
           ncomp = best_pls_c,
           maxiter = max_iter,
-          tol = tol, 
+          tol = tol,
           algorithm = algorithm
         )
       } else {
@@ -237,13 +237,13 @@ pls_cv <- function(x, y, ncomp,
           scale = scale,
           ncomp = ncomp,
           maxiter = max_iter,
-          tol = tol, 
+          tol = tol,
           algorithm = algorithm
         )
       }
     }
   }
-  
+
   if (method == "wapls" & retrieve & !tune) {
     val$compweights <- cv_results$compweights
     val$cv_results <- data.table(
@@ -254,8 +254,8 @@ pls_cv <- function(x, y, ncomp,
       rmse_sd_cv = sd(cv_results$rmse_seg),
       r2_cv = mean(cv_results$rsq_seg)
     )
-    
-    
+
+
     if (!is.null(weights)) {
       x <- sweep(x, 1, weights, "*") ### MODIFIED
       y <- y * weights
@@ -266,41 +266,41 @@ pls_cv <- function(x, y, ncomp,
       scale = scale,
       ncomp = ncomp,
       maxiter = max_iter,
-      tol = tol, 
+      tol = tol,
       algorithm = algorithm
     )
   }
-  
+
   if (method == "wapls" & retrieve & tune) {
     val$cv_results <- data.table(search_grid,
-                                 rmse_cv = rowMeans(cv_results$rmse_seg),
-                                 st_rmse_cv = rowMeans(cv_results$st_rmse_seg),
-                                 rmse_sd_cv = get_col_sds(t(cv_results$rmse_seg)),
-                                 r2_cv = rowMeans(cv_results$rsq_seg)
+      rmse_cv = rowMeans(cv_results$rmse_seg),
+      st_rmse_cv = rowMeans(cv_results$st_rmse_seg),
+      rmse_sd_cv = get_col_sds(t(cv_results$rmse_seg)),
+      r2_cv = rowMeans(cv_results$rsq_seg)
     )
     opmls <- which.min(val$cv_results$rmse_cv)
     val$best_pls_c$min_component <- val$cv_results$min_component[opmls]
     val$best_pls_c$max_component <- val$cv_results$max_component[opmls]
     val$compweights <- cv_results$compweights
-    
-    
+
+
     if (!is.null(weights)) {
       x <- sweep(x, 1, weights, "*")
       y <- y * weights
     }
-    
+
     tmp_compweights <- val$compweights[val$best_pls_c$min_component:val$best_pls_c$max_component]
     val$compweights[] <- 0
     tmp_compweights <- tmp_compweights / sum(tmp_compweights)
     val$compweights[val$best_pls_c$min_component:val$best_pls_c$max_component] <- tmp_compweights
-    
+
     val$models <- opls_get_basics(
       X = x,
       Y = y,
       scale = scale,
       ncomp = ncomp,
       maxiter = max_iter,
-      tol = tol, 
+      tol = tol,
       algorithm = algorithm
     )
   }
@@ -376,9 +376,10 @@ get_wapls_weights <- function(pls_model, original_x, type = "w1", new_x = NULL,
 #' execute a local prediction for the mbl function based on a list of neighbors.
 #' Not valid for local dissmilitary (e.g. for ortho_diss(...., .local = TRUE))
 #' @param Xr the Xr matrix in mbl.
-#' @param Xu the Xu matrix in mbl.
+#' @param Xu the Xu matrix in mbl. Default \code{NULL}. If not provided, the 
+#' function will iterate for each \code{\{Yr, Xr\}} to get the respective neighbors.
 #' @param Yr the Yr matrix in mbl.
-#' @param Yu the Yu matrix in mbl.
+#' @param Yu the Yu matrix in mbl. Default \code{NULL}. 
 #' @param diss_usage a character string indicating if the dissimilarity data
 #' will be used as predictors ("predictors") or not ("none").
 #' @param neighbor_indices a matrix with the indices of neighbors of every Xu
@@ -392,21 +393,22 @@ get_wapls_weights <- function(pls_model, original_x, type = "w1", new_x = NULL,
 #' itemize{
 #' \item{ith_xr:}{ the Xr data of the neighbors for the ith observation (if
 #' \code{diss_usage = "predictors"}, this data is combined with the local
-#' dissmilarity scores of the neighbors of Xu)}
+#' dissmilarity scores of the neighbors of Xu (or Xr if Xu was not provided))}
 #' \item{ith_yr:}{ the Yr data of the neighbors for the ith observation}
-#' \item{ith_xu:}{ the ith Xu observation (if \code{diss_usage = "predictors"},
-#' this data is combined with the local dissmilarity scores to its Xr neighbors}
-#' \item{ith_yu:}{ the ith Yu observation}
+#' \item{ith_xu:}{ the ith Xu observation (or Xr if Xu was not provided). 
+#' If \code{diss_usage = "predictors"}, this data is combined with the local 
+#' dissmilarity scores to its Xr neighbors.}
+#' \item{ith_yu:}{ the ith Yu observation (or Yr observation if Xu was not provided).}
 #' \item{ith_neigh_diss:}{ the dissimilarity scores of the neighbors for the ith
-#' observation}
-#' \item{ith_group:}{ the group labels for ith_xr}
-#' \item{n_k:}{ the number of neighbors}
+#' observation.}
+#' \item{ith_group:}{ the group labels for ith_xr.}
+#' \item{n_k:}{ the number of neighbors.}
 #' }
 #' @details isubset will look at the order of knn in each col of D and
 #' re-organize the rows of x accordingly
 #' @author Leonardo Ramirez-Lopez
 #' @keywords internal
-ith_mbl_neighbor <- function(Xr, Xu, Yr, Yu = NULL,
+ith_mbl_neighbor <- function(Xr, Xu = NULL, Yr, Yu = NULL,
                              diss_usage = "none",
                              neighbor_indices,
                              neighbor_diss = NULL,
@@ -415,8 +417,17 @@ ith_mbl_neighbor <- function(Xr, Xu, Yr, Yu = NULL,
   k_neighbors <- colSums(!is.na(neighbor_indices))
   iter_k_neighbors <- iter(k_neighbors, by = "cell")
   iter_neighbors <- iter(neighbor_indices, by = "col")
-  iter_xu <- iter(Xu, by = "row")
-  iter_yu <- iter(Yu, by = "cell")
+  
+  if (is.null(Xu)) {
+    iter_xu <- iter(Xr, by = "row")
+    iter_yu <- iter(Yr, by = "cell")
+    iterate_yu <- TRUE
+  } else {
+    iter_xu <- iter(Xu, by = "row")
+    iter_yu <- iter(Yu, by = "cell")
+    iterate_yu <- !is.null(Yu)
+  }
+  
   neighbor_diss <- t(neighbor_diss)
   iter_xr_xu_diss <- iter(neighbor_diss, by = "row")
   group
@@ -432,7 +443,7 @@ ith_mbl_neighbor <- function(Xr, Xu, Yr, Yu = NULL,
     ith_xu <- nextElem(iter_xu)
     ith_neigh_diss <- nextElem(iter_xr_xu_diss)[, ith_ks, drop = FALSE]
 
-    if (!is.null(Yu)) {
+    if (iterate_yu) {
       ith_yu <- nextElem(iter_yu)
     } else {
       ith_yu <- NULL
@@ -567,8 +578,6 @@ get_ith_local_neighbors <- function(ith_xr, ith_xu, ith_yr, ith_yu = NULL,
     )
   )
 
-
-
   if (is_predictors) {
     indx_xu <- ncol(local_diss$dissimilarity)
 
@@ -646,7 +655,7 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
   }
 
   if (any(get_col_sds(x) == 0)) {
-    warning("One of the variables has zero variance. Data will not be scale")
+    warning("Variables with zero variance. Data will not be scaled")
     scale <- FALSE
   }
 
@@ -655,7 +664,7 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
   } else {
     algorithm <- "pls"
   }
-    
+
 
   results <- cv_val <- pred <- NULL
   if (pred_method == "gpr") {
@@ -668,7 +677,7 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         p = p,
         number = number,
         group = group,
-        retrieve = "final_model", 
+        retrieve = "final_model",
         seed = seed
       )
       fit <- cv_val$model
@@ -710,8 +719,8 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         retrieve = TRUE,
         tune = tune,
         max_iter = pls_max_iter,
-        tol = pls_tol, 
-        modified = modified, 
+        tol = pls_tol,
+        modified = modified,
         seed = seed
       )
       fit <- cv_val$models
@@ -725,7 +734,7 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         scale = scale,
         ncomp = pls_c,
         maxiter = pls_max_iter,
-        tol = pls_tol, 
+        tol = pls_tol,
         algorithm = algorithm
       )
       ncomp <- pls_c
@@ -757,8 +766,8 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         retrieve = TRUE,
         tune = tune,
         max_iter = pls_max_iter,
-        tol = pls_tol, 
-        modified = modified, 
+        tol = pls_tol,
+        modified = modified,
         seed = seed
       )
 
@@ -786,7 +795,7 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         scale = scale,
         ncomp = pls_c[[2]],
         maxiter = pls_max_iter,
-        tol = pls_tol, 
+        tol = pls_tol,
         algorithm = algorithm
       )
 
@@ -839,7 +848,7 @@ gaussian_pr_cv <- function(x,
                            number = 10,
                            group = NULL,
                            noise_variance = 0.001,
-                           retrieve = c("final_model", "none"), 
+                           retrieve = c("final_model", "none"),
                            seed = NULL) {
 
   ## Create the resampling groups
@@ -848,7 +857,7 @@ gaussian_pr_cv <- function(x,
     p = p,
     number = number,
     group = group,
-    replacement = FALSE, 
+    replacement = FALSE,
     seed = seed
   )
 
