@@ -21,14 +21,14 @@
 #' @keywords internal
 
 sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, seed = NULL) {
-  
+
   ## If the percentage of samples to build the hold_in subset is below 50% of
   ## the total number of samples, the selection is based on the number of samples
   ## to retain.
   ## On the other hand, if the percentage of samples to build the hold_in subset
   ## is equal or above 50% of the total number of samples, the selection is based
   ## on the number of samples to exclude.
-  
+
   # Account for machine precision for calculating the number of samples to take
   # later. The reason is that the floor function can return wrong results, e.g.
   # floor((1 - 0.9) * 50) returns (incorrectly) 4. The following way to round
@@ -38,7 +38,7 @@ sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, s
     8,
     .Machine$sizeof.longdouble - 2
   )
-  
+
   if (p < 0.5) {
     p_to_sample <- round(p, machine_precision)
     do_sampling_for <- "calibration"
@@ -46,34 +46,34 @@ sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, s
     p_to_sample <- round(1 - p, machine_precision)
     do_sampling_for <- "validation"
   }
-  
+
   # Stratified sampling if no group is given
   if (is.null(group)) {
     # Number of samples that must be sampled
     nv <- floor(p_to_sample * nrow(y))
     nv <- ifelse(nv < 1, 1, nv)
-    
+
     # Calculate the number of samples in the validation and calibration sets,
     # respectively.
-    n_val = ifelse(p >= 0.5, nv, nrow(y) - nv)
-    n_cal = ifelse(replacement, nrow(y), nrow(y) - n_val)
-    
+    n_val <- ifelse(p >= 0.5, nv, nrow(y) - nv)
+    n_cal <- ifelse(replacement, nrow(y), nrow(y) - n_val)
+
     # Compute the strata, including how many samples must be taken in each of them.
     strata_category <- optim_sample_strata(
       y = y,
       n = nv
     )
-    
+
     calibration_indices <- matrix(NA, n_cal, number)
     validation_indices <- matrix(NA, n_val, number)
-    
+
     colnames(calibration_indices) <- colnames(validation_indices) <- paste0(
       "Resample_",
       seq(1:number)
     )
     rownames(calibration_indices) <- paste0("index_", seq(1:n_cal))
     rownames(validation_indices) <- paste0("index_", seq(1:nrow(validation_indices)))
-    
+
     # The seed is set both here and if there is a group given. Alternatively,
     # one could set the seed before the if(is.null(group)) statement. However,
     # the way it is done now should ensure that future changes to the code do not
@@ -81,7 +81,7 @@ sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, s
     if (!is.null(seed)) {
       set.seed(seed)
     }
-    
+
     # Obtain the samples drawn from the calculated strata for a total number of
     # 'number' times.
     for (jj in 1:number) {
@@ -116,22 +116,22 @@ sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, s
       INDEX = y_groups$group,
       FUN = mean
     )
-    
+
     # Number of groups that must be sampled
     nv <- floor(p_to_sample * n_levels)
     nv <- ifelse(nv < 1, 1, nv)
-    
+
     # Calculate the number of samples in the validation and calibration sets,
     # respectively.
-    n_val = ifelse(p >= 0.5, nv, nrow(y) - nv)
-    n_cal = ifelse(replacement, nrow(y), nrow(y) - n_val)
-    
+    n_val <- ifelse(p >= 0.5, nv, nrow(y) - nv)
+    n_cal <- ifelse(replacement, nrow(y), nrow(y) - n_val)
+
     # Compute the strata, including how many groups must be taken in each of them.
     strata_category <- optim_sample_strata(
       y = aggregated_y,
       n = nv
     )
-    
+
     calibration_groups_indices <- validation_groups_indices <- NULL
     # The seed is set both here and if there is no group given. Alternatively,
     # one could set the seed before the if(is.null(group)) statement. However,
@@ -140,7 +140,7 @@ sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, s
     if (!is.null(seed)) {
       set.seed(seed)
     }
-    
+
     # Obtain the groups drawn from the calculated strata for a total number of
     # 'number' times.
     for (jj in 1:number) {
@@ -152,24 +152,24 @@ sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, s
         sampling_for = do_sampling_for,
         replacement = replacement
       )
-      
+
       # Since we sampled based on the group levels, we must convert these levels
       # back to the sample indices.
       if (replacement) {
         sel_sample_indices <- lapply(1:length(strata_samples$calibration),
-                                     FUN = function(ith, full_groups, cal_groups) {
-                                       ## this equivalent to extract from
-                                       ## y_groups$original_order
-                                       which(full_groups %in% cal_groups[ith])
-                                     },
-                                     full_groups = y_groups$group,
-                                     cal_groups = gr_levels[strata_samples$calibration]
+          FUN = function(ith, full_groups, cal_groups) {
+            ## this equivalent to extract from
+            ## y_groups$original_order
+            which(full_groups %in% cal_groups[ith])
+          },
+          full_groups = y_groups$group,
+          cal_groups = gr_levels[strata_samples$calibration]
         )
         sel_sample_indices <- do.call("c", sel_sample_indices)
       } else {
         sel_sample_indices <- y_groups$original_order[y_groups$group %in% gr_levels[strata_samples$calibration]]
       }
-      
+
       calibration_groups_indices[[jj]] <- sel_sample_indices
       validation_groups_indices[[jj]] <- y_groups$original_order[y_groups$group %in% gr_levels[strata_samples$validation]]
     }
@@ -187,29 +187,29 @@ sample_stratified <- function(y, p, number, group = NULL, replacement = FALSE, s
     if (!is.null(seed)) {
       set.seed(seed + 1)
     }
-    # Groups can be of different sizes, so the 
+    # Groups can be of different sizes, so the
     for (jj in 1:number) {
       ## randomly select the replacements for the missing indexes
       ## replacements are necessary!
       n_cal_missing <- max(lengths_list_calibration_indices) - lengths_list_calibration_indices[jj]
       jj_add_list_calibration_indices <- sample(calibration_groups_indices[[jj]],
-                                                size = n_cal_missing,
-                                                replace = TRUE
+        size = n_cal_missing,
+        replace = TRUE
       )
-      
+
       n_val_missing <- max(lengths_list_validation_indices) - lengths_list_validation_indices[jj]
       jj_add_list_validation_indices <- sample(validation_groups_indices[[jj]],
-                                               size = n_val_missing,
-                                               replace = TRUE
+        size = n_val_missing,
+        replace = TRUE
       )
-      
+
       calibration_indices[1:lengths_list_calibration_indices[jj], jj] <- calibration_groups_indices[[jj]]
       validation_indices[1:lengths_list_validation_indices[jj], jj] <- validation_groups_indices[[jj]]
       calibration_indices[-(1:lengths_list_calibration_indices[jj]), jj] <- jj_add_list_calibration_indices
       validation_indices[-(1:lengths_list_validation_indices[jj]), jj] <- jj_add_list_validation_indices
     }
   }
-  
+
   list(
     hold_in = calibration_indices,
     hold_out = validation_indices
@@ -229,11 +229,11 @@ get_sample_strata <- function(y, n = NULL, probs = NULL) {
   if (!is.null(n) & !is.null(probs)) {
     stop("both n and probs have been passed to the function, only one of them can be accepted")
   }
-  
+
   if (!is.null(n)) {
     probs <- seq(0, 1, length = (n + 1))
   }
-  
+
   y_strata <- unique(
     quantile(
       y,
@@ -241,7 +241,7 @@ get_sample_strata <- function(y, n = NULL, probs = NULL) {
       names = FALSE
     )
   )
-  
+
   strata_labels <- 1:max(length(y_strata) - 1, 1)
   y_cuts <- cut(
     y,
@@ -250,7 +250,7 @@ get_sample_strata <- function(y, n = NULL, probs = NULL) {
     include.lowest = TRUE,
     right = FALSE # Use left closed intervals for compatibility with Julia code
   )
-  
+
   strata_category <- data.table(
     original_order = 1:length(y),
     strata = y_cuts
@@ -273,7 +273,7 @@ optim_sample_strata <- function(y, n) {
   n_strata <- length(unique(sample_strata$strata))
   iter <- 1
   table_strata <- table(sample_strata$strata)
-  
+
   # If the number of strata is lower than number of samples, we must correct the
   # strata such that we have the correct number of samples in each stratum.
   #
@@ -287,24 +287,24 @@ optim_sample_strata <- function(y, n) {
     new_min_samples_per_strata <- 2 # There must be at least 2 samples per strata
     while (keep_going) {
       # n_vec <- 1:new_n
-      
+
       # We have to increase the minimum number of samples per strata as we decrease
       # the number of strata. We increase by two; one for the drawing the sample
       # set, one for drawing the replacement set.
       new_min_samples_per_strata <- new_min_samples_per_strata + 2
-      
+
       # s_missing <- n_vec[-as.numeric(names(table_strata))]
       # v_missing <- rep(0, length(s_missing))
       # names(v_missing) <- s_missing
       # table_strata <- c(table_strata, v_missing)
-      
+
       # Assign a new number of strata
       new_n <- ceiling(n / (iter + 1))
-      
+
       # Get the new sample strata for the reduced number of strata
       sample_strata <- get_sample_strata(y, new_n)
       table_strata <- table(sample_strata$strata)
-      
+
       # Check that each strata contains at least the previously set minimum
       # number of samples. Must be strictly less; otherwise eliminate randomness
       # from the sampling.
@@ -313,13 +313,13 @@ optim_sample_strata <- function(y, n) {
       condition_2 <- length(table_strata) == new_n
       # Check that we can further reduce the number of strata without issue
       condition_3 <- new_min_samples_per_strata >= n
-      
+
       if ((condition_1 & condition_2) | condition_3) {
         break
       }
       iter <- iter + 1
     }
-    
+
     # Describes how many samples to get for each strata. We start by assigning
     # the same number of samples in each stratum. For some strata, we then reduce
     # this number, as there will (in most cases) more total samples in all strata
@@ -386,7 +386,7 @@ get_samples_from_strata <- function(y,
   if (val_replacement) {
     samples_per_strata$samples_to_get <- 2 * samples_per_strata$samples_to_get
   }
-  
+
   # This functions draws samples from a given single stratum. We do not draw with
   # replacement here, as the replacements are done manually later.
   # For selecting the replacement samples in cases where a strata has only one
@@ -400,42 +400,42 @@ get_samples_from_strata <- function(y,
   }
   max_samples <- max(samples_per_strata$samples_to_get)
   vec_samples <- rep(NA, max_samples)
-  
+
   # Here, we draw samples from each strata. In case of sampling for validation
   # with replacement, we draw twice as many samples compared to the other methods.
   # These drawn samples will be split into two sets, one denoting the validation
   # indices, whereas the second one describes the replacement indices that replace
   # the drawn samples.
   strata_samples <- lapply(levels(strata),
-                           FUN = function(strata,
-                                          original_order,
-                                          samples_per_strata,
-                                          vec_samples,
-                                          replacement,
-                                          ii) {
-                             ith_n <- samples_per_strata$samples_to_get[samples_per_strata$strata == ii]
-                             ith_set <- original_order[which(strata == ii)]
-                             ith_sel <- get_random_sample(ith_set, ith_n)
-                             if (replacement) {
-                               # Split the drawn samples into the two above described sets. The sets are
-                               # both contained in vec_samples, but these vectors will be split in half
-                               # later. Note that we must take special care with the indexing here, to
-                               # ensure that the samples are put into the correct set.
-                               ln <- (length(ith_sel) / 2)
-                               vec_samples[1:ln] <- ith_sel[1:ln]
-                               vec_samples[(length(vec_samples) - ln + 1):length(vec_samples)] <- ith_sel[-(1:ln)]
-                             } else {
-                               vec_samples[1:length(ith_sel)] <- ith_sel
-                             }
-                             vec_samples
-                           },
-                           strata = strata,
-                           original_order = original_order,
-                           samples_per_strata = samples_per_strata,
-                           vec_samples = vec_samples,
-                           replacement = val_replacement
+    FUN = function(strata,
+                   original_order,
+                   samples_per_strata,
+                   vec_samples,
+                   replacement,
+                   ii) {
+      ith_n <- samples_per_strata$samples_to_get[samples_per_strata$strata == ii]
+      ith_set <- original_order[which(strata == ii)]
+      ith_sel <- get_random_sample(ith_set, ith_n)
+      if (replacement) {
+        # Split the drawn samples into the two above described sets. The sets are
+        # both contained in vec_samples, but these vectors will be split in half
+        # later. Note that we must take special care with the indexing here, to
+        # ensure that the samples are put into the correct set.
+        ln <- (length(ith_sel) / 2)
+        vec_samples[1:ln] <- ith_sel[1:ln]
+        vec_samples[(length(vec_samples) - ln + 1):length(vec_samples)] <- ith_sel[-(1:ln)]
+      } else {
+        vec_samples[1:length(ith_sel)] <- ith_sel
+      }
+      vec_samples
+    },
+    strata = strata,
+    original_order = original_order,
+    samples_per_strata = samples_per_strata,
+    vec_samples = vec_samples,
+    replacement = val_replacement
   )
-  
+
   # Put all drawn samples into one single matrix
   strata_samples <- do.call("rbind", strata_samples)
   if (val_replacement) {
@@ -454,8 +454,8 @@ get_samples_from_strata <- function(y,
     # In any other case, we can simply put them into a matrix
     strata_samples <- as.matrix(sort(strata_samples))
   }
-  
-  
+
+
   if (sampling_for == "validation") {
     if (replacement) {
       # Ensure that the validation and replacement sets are distinct.
@@ -474,8 +474,8 @@ get_samples_from_strata <- function(y,
     keep <- original_order[!original_order %in% strata_samples[, 1]]
     exclude <- as.vector(sort(strata_samples[, 1]))
   }
-  
-  
+
+
   if (sampling_for == "calibration") {
     # For calibration, we can already use the constructed calibration and validation
     # sets. In case of sampling with replacements, the replacement indices are
@@ -496,7 +496,7 @@ get_samples_from_strata <- function(y,
   # if (sampling_for == "calibration") {
   #   keep <- strata_samples[, 1]
   #   exclude <- original_order[!original_order %in% keep]
-  #   if (replacement) {     
+  #   if (replacement) {
   #     (length(original_order) +length(keep)) / (length(original_order) - length(keep))
   #     ip <-
   #      replacement_indices <-  sample_stratified(y = y[keep, ], p = ip, number = 1, seed = seed)
@@ -505,10 +505,10 @@ get_samples_from_strata <- function(y,
   #     replacement_indices <- NULL
   #   }
   # }
-  
+
   # Add replacement indices to the calibration set.
   keep <- sort(as.vector(c(keep, replacement_indices)))
-  
+
   strata_samples <- list(
     calibration = keep,
     validation = exclude
@@ -527,17 +527,17 @@ get_quartile_samples <- function(y, n) {
     return(NULL)
   }
   nn <- floor(n / length(quartile_probs))
-  
+
   if (nn < 1) {
     nn <- 1
   }
-  
+
   ## samples per stratum
   ssampling <- data.frame(
     stratum = sort(as.numeric(unique(strata$strata))),
     n = nn
   )
-  
+
   # I think this is impossible to reach.
   # if (sum(ssampling$n) > n) {
   #   repl <- length(ssampling$stratum) < sum(ssampling$n) - n
@@ -554,7 +554,7 @@ get_quartile_samples <- function(y, n) {
   #     ssampling$n[i] <- ssampling$n[ssampling$stratum == i] - i_add
   #   }
   # }
-  
+
   if (sum(ssampling$n) < n) {
     repl <- length(ssampling$stratum) < n - sum(ssampling$n)
     adds <- sample(ssampling$stratum, n - sum(ssampling$n), replace = repl)
@@ -572,31 +572,31 @@ get_quartile_samples <- function(y, n) {
       ssampling$n[ssampling$stratum == adds] <- ssampling$n[ssampling$stratum == adds] + 1
     }
   }
-  
+
   strat_samples <- lapply(
     ssampling$stratum,
     FUN = function(stratum_data, stratum_n, i) {
       smp <- stratum_data$original_order[stratum_data$strata == i]
       i_n <- stratum_n$n[stratum_n$stratum == i]
       sample(smp,
-             i_n,
-             replace = ifelse(i_n > length(smp), TRUE, FALSE))
+        i_n,
+        replace = ifelse(i_n > length(smp), TRUE, FALSE)
+      )
     },
     stratum_data = strata,
     stratum_n = ssampling
   )
-  
+
   strat_samples <- sort(unlist(strat_samples))
-  
+
   if (length(strat_samples) > n) {
     overs <- length(strat_samples) - n
     strat_samples <- strat_samples[-sample(length(strat_samples), overs)]
   }
-  
+
   if (length(strat_samples) < n) {
     unders <- n - length(strat_samples)
     strat_samples <- c(strat_samples, sort(sample(strat_samples, unders, replace = TRUE)))
   }
   strat_samples
-  
 }
